#!/usr/bin/env python
from __future__ import division
import os, sys, re, getopt
from stat import *

class fatalError(Exception):
    def __init__(self,errMsg):
        self.data=errMsg
    def __str__(self):
        print >> sys.stderr, `self.data`
        exit

class parseError(Exception):
    def __init__(self,errMsg):
        self.data=errMsg
    def __str__(self):
        print >> sys.stderr, "parseError: "+`self.data`

class templateError(Exception):
    def __init__(self,errMsg):
        self.data=errMsg
    def __str__(self):
        print >> sys.stderr, "templateError: "+`self.data`

class nodeClass:
    def __init__(self):
        self.name=None
        self.parent=None
        self.filename=None
        self.children=[]

inFileExt=""
outFileExt=".html"
clobber=False
nodes={}
files={}
fileHasNode={}
traceLevel=0

template=['<div class="left-content">',
          '<p>',
          '<h1>',
          '<a href="index.html">Andrew Mobbs</a>',
          '</h1>',
          'MENU',
          '</p>',
          '</div>',
          '<div class="right-content">',
          'BODY',
          '</div>']

def walktree(dir, callback):
    '''recursively descend the directory rooted at dir,
    calling the callback function for each regular file'''

    for f in os.listdir(dir):
        pathname = '%s/%s' % (dir, f)
        mode = os.stat(pathname)[ST_MODE]
        if S_ISDIR(mode):
            walktree(pathname, callback)
        elif S_ISREG(mode):
            callback(pathname)
        else:
            print 'Skipping %s' % pathname

def parseFile(filename):
    '''Builds tree of files'''

    if not filename.endswith(inFileExt):
        return

    if filename.startswith("./"):
        filename=filename[2:]

    try:
        fh=file(filename,"r")
    except IOError:
        print >> sys.stderr,"Error opening "+str(filename)
        return
    nodeName=None
    nodeParent=None
    htmlFound=False
    
    # Parse file, checking that it's HTML (no check for validity) and looking
    # for tags for document name and parent
    try:
        for line in fh.xreadlines():
            if line.find("<!-- NAME:") >= 0:
                match=re.search(r"NAME:\s*(.*)\s*-->",line)
                if match == None:
                    raise parseError("Error matching node name in : "+str(line))
                nodeName=match.group(1)
            if line.find("<!-- PARENT:") >= 0:
                match=re.search(r"PARENT:\s*(.*)\s*-->",line)
                if match == None:
                    raise parseError("Error matching node parent in : "+str(line))
                nodeParent=match.group(1)
            if line.lower().find("<html") >= 0:
                htmlFound=True
            if nodeParent != None and nodeName != None and htmlFound == True:
                files[filename]=True
                break
    finally:
        fh.close()
        
    # Report on validity
    if htmlFound is False:
        print >> sys.stderr,str(filename)+" doesn't appear to be HTML"
        return

    if nodeName == None or nodeParent == None:
        print >> sys.stderr,"Couldn't find both name and parent in : "+str(filename)
        return

    # Build Tree
    # Get node record if it exists, else create it
    if nodeName in nodes:
        node=nodes[nodeName]
    else:
        node=nodeClass()
        node.name=nodeName
        nodes[nodeName]=node

    node.parent=nodeParent
    node.filename=filename
    fileHasNode[filename]=node

    # Get parent if it exists, else create it
    if nodeParent in nodes:
        parent=nodes[nodeParent]
    else:
        parent=nodeClass()
        parent.name=nodeParent
        nodes[nodeParent]=parent
        
    # Add node to parent's list of children
    parent.children.append(nodeName)

def fixFileExtention(filename):
    '''Strips input file extention (if known) and adds output file extention'''
    outFilename=filename[0:filename.rfind(inFileExt)]+outFileExt
    return outFilename

def formatPage(inFilename):

    if inFilename.startswith("./"):
        inFilename=inFilename[2:]

    # Bomb if we're not supposed to be dealing with this file
    if inFilename not in files:
        if traceLevel > 0:
            print >> sys.stderr,"format filename not in list"
        return

    # Attempt to open input file
    try:
        inFile=file(inFilename,"r")
    except IOError:
        print >> sys.stderr,"Error opening "+str(inFilename)
        return

    outFilename=fixFileExtention(inFilename)

    # If we're not supposed to clobber existing files, check for existance
    if clobber is False:
        outFileExists=True
        try:
            os.stat(outFilename)
        except OSError:
            outFileExists=False
        if outFileExists == True:
            print >> sys.stderr,outFilename+" already exists, not overwriting."
            return
        
    # Open output file
    try:
        outFile=file(outFilename,"w+")
    except IOError:
        print >> sys.stderr,"Couldn't open "+outFilename+" for writing."
        return
    
    bodyStartFound=False
    bodyEndFound=False
    
    for line in inFile.xreadlines():

        # Need to get end of the template written just before the </body> tag
        if line.lower().find("</body") >= 0:
            bodyendFound=True
            startPos=template.index("BODY")+1
            for templateLine in template[startPos:]:
                print >> outFile,templateLine

        # Copy line from input file
        outFile.write(line)

        # Write start of template just after <body> tag
        if line.lower().find("<body") >= 0:
            bodyStartFound=True
            for templateLine in template:
                if templateLine == "BODY":
                    break
                if templateLine == "MENU":
                    writeMenu(inFile,outFile)
                    continue
                print >> outFile,templateLine

    inFile.close()
    outFile.close()

def writeMenu(inFile,outFile):
    try:
        thisNode=fileHasNode[inFile.name]
    except KeyError:
        raise fatalError("Internal Error: No node for "+inFile.name+" in writeMenu()")

    #Find the root node for this entry
    rootNode=thisNode
    treeList=[]
    while rootNode.parent != rootNode.name:
        if rootNode.name in treeList:
            raise fatalError("Cyclic graph detected from node "+thisNode.name)
        treeList.append(rootNode.name)
        rootNode=nodes[rootNode.parent]

    dumpTree(outFile,rootNode,treeList)


def dumpTree(outFile,currentNode,treeList):
    if not currentNode.children:
        return
    print >> outFile,"<ul>"
    for nodeName in currentNode.children:
        if nodeName == currentNode.name:
            continue
        filename=fixFileExtention(nodes[nodeName].filename)
        print >> outFile,'<li><a href="%(filename)s">%(nodeName)s</a></li>' % vars()
        if nodeName in treeList:
            treeList.remove(nodeName)
            dumpTree(outFile,nodes[nodeName],treeList)
    print >> outFile,"</ul>"
            
def validateTemplate():
    try:
        try:
            menuPos=template.index("MENU")
            bodyPos=template.index("BODY")
        except ValueError:
            raise templateError("Missing required element")
        if menuPos > bodyPos:
            raise templateError("Elements in wrong order")
    except templateError:
        sys.exit(2)

def loadTemplate(templateFile):
    global template

    try:
        tFile=file(templateFile)
    except IOError:
        fatalError("Could not open template file: "+templateFile)
    template=[]
    for line in tFile.xreadlines():
        template.append(line.strip())
        
        
def main():
    global inFileExt,outFileExt,clobber
    try:
        opts,args=getopt.getopt(sys.argv[1:], "hci:o:t:")
    except getopt.GetoptError:
        usage()
        sys.exit(2)

    for opt, arg in opts:
        if opt == "-h":
            usage()
            sys.exit()
        if opt == "-c":
            clobber=True
        if opt == "-i":
            if not arg.startswith("."):
                arg="."+arg
            inFileExt=arg                
        if opt == "-o":
            if not arg.startswith("."):
                arg="."+arg
            outFileExt=arg
        if opt == "-t":
            loadTemplate(arg)
    validateTemplate()
    for path in args:
        walktree(path,parseFile)
    for path in args:
        walktree(path,formatPage)
    if traceLevel > 0:
        for key in nodes:
            print key,nodes[key].parent,nodes[key].filename,nodes[key].children

            
def usage():
    print 'Usage: '+sys.argv[0]
    print '-h          Display this help'
    print '-c          Overwrite output files (Clobber) - Default is not to'
    print '-i <ext>    Set input file extention to <ext> - Default all files'
    print '-o <ext>    Set output file extention to <ext> - Default "html"'
    print '-t <file>   Load template from file'
    print
    print 'Generates HTML tree menus.'
    print 'Scans the input files, which must be HTML, for tags:'
    print '<!-- NAME:[name] -->'
    print '<!-- PARENT:[name] -->'
    print 'Root nodes are indicated by having the PARENT name be the same'
    print 'as the document name.'
    print
    print 'Template file format is HTML to include immediately after'
    print 'the <body> tag, with "MENU" where menu DIV should'
    print 'go, and "BODY" where to place the original body text.' 
    print 'The template should include all the necessary HTML to format'
    print 'the menu.'

if __name__ == '__main__':
    main()
